local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage", "GatewayManage")

if ("GET" == request_method) then
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end

    local driver_name = args["DriverName"]
    local pk = args["ProductKey"]
    local dn = args["DeviceName"]

    if (not (driver_name and pk and dn)) then
        utils.error_exit(400, "invalid parameter")
    end

    if ("gateway_monitor" == driver_name and "edge" == dn) then
        pk, dn, _, _, _ = utils.get_gw_config()
        local info = {
            ProductKey = pk,
            DeviceName = dn,
            DeviceLocalId = dn,
            ProductName = "",
            DriverName = "gateway_monitor",
            DeviceTags = {},
            DeviceConfig = {},
            LocalState = "Online",
        }
        local state, err = utils.get_core_connection_state()
        if not err then
            if state["connected"] then
                info["CloudState"] = "Online"
                info["LastCloudOnlineTime"] = tostring(state["timestamp"])
                info["LastLocalOnlineTime"] = info["LastCloudOnlineTime"]
                info["LastLinkTime"] = info["LastLocalOnlineTime"]
            else
                info["CloudState"] = "Offline"
            end
        end
        utils.error_exit(200, json.encode(info))
    end

    local ok, dev_info = utils_device.get_device_info(pk, dn)
    if (ok) then
        dev_info["DeviceConfig"] = utils_device.get_device_config(driver_name, pk, dn)
        utils.error_exit(200, json.encode(dev_info))
    else
        utils.error_exit(200, '{}') 
    end

elseif("POST" == request_method) then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    if (nil == args_json) then
        utils.error_exit(400)
    end 

    local args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400)
    end 

    local driver_name = args["DriverName"]
    local pk = args["ProductKey"]
    local dn = args["DeviceName"]
    local tags = args["DeviceTags"]
    local config = args["DeviceConfig"]

    if (not (driver_name and pk and dn and tags and config)) then
        utils.error_exit(400, "invalid parameter")
    end

    if ("string" == type(tags)) then
        tags = {tags} -- convert to table type if @tags is a string type
    end

    if ("string" == type(config)) then
        config = {config}
    end

    if (tags and "" ~= tags) then    
        utils.set_device_tags(pk, dn, tags)
    end

    utils_device.set_device_config(driver_name, pk, dn, config)
    utils.error_exit(200, '{}')
end

utils.error_exit(400, "unsupported request method")
