local const = require("const")
local json = require("json")
local utils = require("utils2")

local shadow = {}

function shadow:new(o, product, device)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    self.product = product
    self.device = device
    return o
end

function shadow:get_properties(identifiers)
    local params = { params = identifiers }
    local id = self.product .. "_" .. self.device
    local well_known_name = const.cmp.wkn_prefix .. id
    local path = const.cmp.obj_prefix .. id
    local interface = const.cmp.inf_prefix .. id
    local ok, ret, err = utils.dbus_call_edge(
            well_known_name, path, interface,
            "callServices", "get", json.encode(params))
    if not ok then
        return false, nil, err
    end

    local res
    ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return false, nil, "not a JSON object retrieved from thing"
    end
    if res["code"] ~= 0 then
        return false, nil, res["message"]
    end
    return true, res["params"], nil
end

function shadow:set_properties(properties)
    local params = { params = properties }
    local id = self.product .. "_" .. self.device
    local well_known_name = const.cmp.wkn_prefix .. id
    local path = const.cmp.obj_prefix .. id
    local interface = const.cmp.inf_prefix .. id
    local ok, ret, err = utils.dbus_call_edge(
            well_known_name, path, interface,
            "callServices", "set", json.encode(params))
    if not ok then
        return false, nil, err
    end

    local res
    ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return nil, "not a JSON object retrieved from thing"
    end
    if res["code"] ~= 0 then
        return false, nil, res["message"]
    end
    return true, res["params"], nil
end

function shadow:call_service(name, args)
    local params = { params = args }
    local id = self.product .. "_" .. self.device
    local well_known_name = const.cmp.wkn_prefix .. id
    local path = const.cmp.obj_prefix .. id
    local interface = const.cmp.inf_prefix .. id
    local ok, ret, err = utils.dbus_call_edge(
            well_known_name, path, interface,
            "callServices", name, json.encode(params))
    if not ok then
        return false, nil, err
    end

    local res
    ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return nil, "not a JSON object retrieved from thing"
    end
    if res["code"] ~= 0 then
        return false, nil, res["message"]
    end
    local res_params = res["params"]
    if res_params then
        if res_params["code"] ~= 0 then
            return false, nil, res_params["message"]
        end
        return true, res_params["data"], nil
    end
    return true, nil, nil
end

return shadow